# Copyright (c) 2012-2013 gocept gmbh & co. kg
# See also LICENSE.txt

import datetime
import os.path
import pkg_resources
import pkginfo
import shutil
import sys


AUTOSUMMARY_OUTPUT = './_api/'


def set_defaults(egg=True):
    _confpy = sys._getframe(1).f_locals

    author = 'gocept'
    if egg:
        _dist = pkginfo.Develop(os.path.join('..', 'src'))
        project = _dist.name
        author = _dist.author

        release = _dist.version
        version = []
        for x in release:
            try:
                version.append(str(int(x)))
            except ValueError:
                break
        version = '.'.join(version)

    _year = datetime.date.today().year
    _year_started = _confpy.get('_year_started', _year)
    if str(_year) != str(_year_started):
        _year = u'%s-%s' % (_year_started, _year)
    copyright = u'%s %s' % (_year, author)

    source_suffix = '.txt'
    master_doc = 'index'

    needs_sphinx = '1.0'
    extensions = [
        'sphinx.ext.autosummary',
        'sphinx.ext.viewcode',
        ]

    autosummary_generate = ['api.txt']
    _autosummary_output = AUTOSUMMARY_OUTPUT

    templates_path = [
        pkg_resources.resource_filename(
            'gocept.package', 'themes/gocept/templates'),
        ]

    html_theme_path = [
        pkg_resources.resource_filename('gocept.package', 'themes')]
    html_theme = 'gocept'

    _default_sidebars = ['globaltoc.html', 'searchbox.html']
    if egg:
        _default_sidebars.insert(0, 'project-links.html')
    html_sidebars = {
        '**': _default_sidebars,
    }

    html_context = {}

    html_logo = pkg_resources.resource_filename(
        'gocept.package', 'themes/gocept/static/gocept.png')
    html_favicon = pkg_resources.resource_filename(
        'gocept.package', 'themes/gocept/static/favicon.ico')
    html_show_sourcelink = False

    for key, value in locals().items():
        if key not in _confpy:
            _confpy[key] = value

    # We use the autosummary extension to build API docs from source code.
    # However, this extension doesn't update the generated docs if the source
    # files change. Therefore, we need to remove the generated stuff before
    # each run. The _autosummary_output variable tells the relative path to
    # the directory that autosummary uses to put its generated files and which
    # we, therefore, need to remove. It must be the same that the autosummary
    # directive in api.txt points to.

    if os.path.isdir(_autosummary_output):
        shutil.rmtree(_autosummary_output)
    elif os.path.exists(_autosummary_output):
        raise RuntimeError('Expected %s to be a directory.' %
                           os.path.abspath(_autosummary_output))
    os.mkdir(_autosummary_output)



# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.package.doc
import gocept.testing.assertion
import os.path
import shutil
import subprocess
import sys
import tempfile
import unittest2 as unittest


class DocBuildEndtoend(unittest.TestCase, gocept.testing.assertion.Ellipsis):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        self.cwd = os.getcwd()
        os.chdir(self.tmpdir)
        self.mkdir('src')
        self.write('setup.py', """\
from setuptools import setup

setup(
    name='testpackage',
    version='1.0',
    author='Author',
    package_dir={'': 'src'},
)
""")
        subprocess.call([sys.executable, 'setup.py', 'egg_info'])
        self.mkdir('doc')
        self.write('doc/api.txt', 'dummy')

    def tearDown(self):
        os.chdir(self.cwd)
        shutil.rmtree(self.tmpdir)

    def mkdir(self, path):
        os.mkdir(os.path.join(self.tmpdir, path))

    def write(self, path, contents):
        with open(os.path.join(self.tmpdir, path), 'w') as f:
            f.write(contents)

    def test_should_generate_documentation(self):
        self.write('doc/conf.py', """\
import gocept.package.sphinxconf
gocept.package.sphinxconf.set_defaults()
""")
        self.write('doc/index.txt', 'foo and bar and qux')
        gocept.package.doc.main(['doc'])
        index_html = os.path.join(self.tmpdir, 'build/doc/index.html')
        self.assertTrue(os.path.isfile(index_html))
        contents = open(index_html).read()
        self.assertEllipsis('...foo and bar and qux...', contents)

    def test_variables_from_confpy_are_available_in_sphinxconf_module(self):
        self.write('doc/conf.py', """\
import gocept.package.sphinxconf

_year_started = 2000
gocept.package.sphinxconf.set_defaults()
        """)
        self.write('doc/index.txt', 'foo and bar and qux')
        gocept.package.doc.main(['doc'])
        index_html = os.path.join(self.tmpdir, 'build/doc/index.html')
        contents = open(index_html).read()
        self.assertEllipsis('...Copyright 2000-2...', contents)

    def test_defaults_from_sphinxconf_should_not_override_confpy(self):
        self.write('doc/conf.py', """\
import gocept.package.sphinxconf

release = '2.0beta'
gocept.package.sphinxconf.set_defaults()
        """)
        self.write('doc/index.txt', 'foo and bar and qux')
        gocept.package.doc.main(['doc'])
        index_html = os.path.join(self.tmpdir, 'build/doc/index.html')
        contents = open(index_html).read()
        self.assertEllipsis('...testpackage v2.0beta...', contents)

    def test_command_line_arguments_are_passed_to_sphinx_build(self):
        self.write('doc/conf.py', """\
import gocept.package.sphinxconf

release = '2.0beta'
gocept.package.sphinxconf.set_defaults()
        """)
        self.write('doc/index.txt', 'foo and bar and qux')
        gocept.package.doc.main(['doc', '-D', 'release=3.1.4'])
        index_html = os.path.join(self.tmpdir, 'build/doc/index.html')
        contents = open(index_html).read()
        self.assertEllipsis('...testpackage v3.1.4...', contents)



# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

import os
import os.path
import pkg_resources
import sys


def main(argv=sys.argv):
    cwd = os.getcwd()
    sphinx_build = pkg_resources.load_entry_point(
        'Sphinx', 'console_scripts', 'sphinx-build')
    argv = ['sphinx-build'] + argv[1:]
    argv += ['-E', os.path.join(cwd, 'doc'), os.path.join(cwd, 'build', 'doc')]
    sphinx_build(argv)



# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

from paste.script.templates import var
import datetime
import os
import os.path
import paste.script.templates
import paste.util.template
import pkginfo
import shutil
import subprocess


class Skeleton(paste.script.templates.Template):

    summary = (
        'Python package with buildout, conforming to conventions at gocept')

    _template_dir = 'skeleton'

    template_renderer = staticmethod(
        paste.util.template.paste_script_template_renderer)

    vars = [
        var("description", "One-line description of the package"),
        var("keywords", "Space-separated keywords/tags"),
    ]

    def underline_double(self, text):
        return '=' * len(text)

    def pre(self, command, output_dir, vars):
        namespace, package = vars['egg'].split('.')
        gocept_package = pkginfo.Installed('gocept.package')
        vars.update(
            namespace=namespace,
            package=package,
            year=datetime.date.today().year,
            underline_double=self.underline_double,
            gocept_package_version=gocept_package.version,
            )

    def post(self, command, output_dir, vars):
        os.rename(os.path.join(output_dir, 'hgignore'),
                  os.path.join(output_dir, '.hgignore'))
        subprocess.call(['hg', 'init', output_dir])
        shutil.move(os.path.join(output_dir, 'hgrc'),
                    os.path.join(output_dir, '.hg'))

        os.rename(os.path.join(output_dir, 'coveragerc'),
                  os.path.join(output_dir, '.coveragerc'))



# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

"""A paste.script template following gocept Python package conventions.
"""

from setuptools import setup, find_packages
import glob
import os.path


def project_path(*names):
    return os.path.join(os.path.dirname(__file__), *names)


setup(
    name='gocept.package',
    version='1.1',

    install_requires=[
        'PasteScript',
        'distribute',
        'pkginfo>=0.9',
        ],

    extras_require={
        'doc': [
            'Sphinx>=1.0',
            ],
        'test': [
            'gocept.testing',
            'unittest2',
            ],
        },

    entry_points={
        'console_scripts': [
            'doc=gocept.package.doc:main',
            ],
        'paste.paster_create_template': [
            'gocept-package = gocept.package.skeleton:Skeleton'
            ],
        },

    author=('Thomas Lotze <tl at gocept dot com> and '
            'Wolfgang Schnerring <ws at gocept dot com>'),
    author_email='mail@gocept.com',
    license='ZPL 2.1',
    url='https://bitbucket.org/gocept/gocept.package/',

    keywords='paste.script paster create template python package sphinx theme',
    classifiers="""\
Environment :: Plugins
Framework :: Paste
Intended Audience :: Developers
License :: OSI Approved :: Zope Public License
Programming Language :: Python
Programming Language :: Python :: 2
Programming Language :: Python :: 2.6
Programming Language :: Python :: 2.7
Programming Language :: Python :: 2 :: Only
"""[:-1].split('\n'),
    description=__doc__.strip(),
    long_description='\n\n'.join(open(project_path(name)).read() for name in (
            'README.txt',
            'HACKING.txt',
            'CHANGES.txt',
            )),

    namespace_packages=['gocept'],
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    data_files=[('', glob.glob(project_path('*.txt')))],
    zip_safe=False,
    )



# Copyright (c) 2012-2013 gocept gmbh & co. kg
# See also LICENSE.txt

import datetime
import gocept.package
import gocept.package.doc
import gocept.package.sphinxconf
import gocept.testing.assertion
import os
import os.path
import paste.script.command
import pkginfo
import shutil
import subprocess
import sys
import tempfile
import time
import unittest2 as unittest


class SkeletonSetUp(unittest.TestCase):

    def setUp(self):
        self.cwd = os.getcwd()
        self.tmpdir = tempfile.mkdtemp()
        os.chdir(self.tmpdir)

    def tearDown(self):
        os.chdir(self.cwd)
        shutil.rmtree(self.tmpdir)

    def expand_template(self):
        try:
            paste.script.command.run(
                'create -t gocept-package gocept.example'.split() + [
                    'description="An example package." ',
                    'keywords="example package"',
                    ])
        except SystemExit:
            pass

    def content(self, rel_path):
        file_path = os.path.join(
            self.tmpdir, 'gocept.example', *rel_path.split('/'))
        self.assertTrue(os.path.isfile(file_path))
        return open(file_path).read()


class Skeleton(SkeletonSetUp, gocept.testing.assertion.Ellipsis):

    def test_expanding_template_creates_files(self):
        self.expand_template()
        self.assertEqual(
            '', self.content('src/gocept/example/tests/__init__.py'))
        self.assertIn(
            str(datetime.date.today().year), self.content('COPYRIGHT.txt'))

    def test_package_has_gocept_package_version_pinned_to_active(self):
        self.expand_template()
        gocept_package = pkginfo.Installed('gocept.package')
        self.assertIn('gocept.package = %s\n' % gocept_package.version,
                      self.content('versions/versions.cfg'))

    def test_hg_init_has_been_run(self):
        self.expand_template()
        self.assertTrue(os.path.isdir(os.path.join('gocept.example', '.hg')))

    def test_setup_py_is_functional(self):
        # paster detects setup.py and creates egg-info from it
        self.expand_template()
        self.assertIn('Name: gocept.example\n',
                      self.content('src/gocept.example.egg-info/PKG-INFO'))

    def test_sphinx_docs_can_be_built(self):
        self.expand_template()
        os.chdir('gocept.example')
        gocept.package.doc.main(['doc'])
        self.assertIn('<html', self.content('build/doc/index.html'))

    def test_project_links_are_fully_expanded_in_sphinx_sidebar(self):
        self.expand_template()
        os.chdir('gocept.example')
        gocept.package.doc.main(['doc'])
        self.assertEllipsis(
            '...<a href="https://bitbucket.org/gocept/gocept.example/">Project home</a>...',
            self.content('build/doc/index.html'))
        self.assertIn(
            '<a href="http://pypi.python.org/pypi/gocept.example/">PyPI</a>',
            self.content('build/doc/index.html'))

    def test_coveragerc_is_renamed_with_dot(self):
        self.expand_template()
        self.assertIn('source = gocept.example', self.content('.coveragerc'))


class Buildout(SkeletonSetUp):

    level = 2

    def setUp(self):
        super(Buildout, self).setUp()
        self.expand_template()
        os.chdir('gocept.example')

    @property
    def gocept_package_dev(self):
        path = gocept.package.__file__
        for _ in xrange(4):
            path = os.path.dirname(path)
        return path

    def buildout(self):
        subprocess.call([sys.executable, 'bootstrap.py'])
        return subprocess.call([
                os.path.join('bin', 'buildout'),
                'buildout:develop+=%s' % self.gocept_package_dev])

    def test_bootstrap_succeeds_using_distribute_by_default(self):
        subprocess.call([sys.executable, 'bootstrap.py'])
        bin_buildout = self.content('bin/buildout')
        self.assertIn(sys.executable, bin_buildout)
        self.assertIn('distribute-', bin_buildout)
        self.assertNotIn('setuptools-', bin_buildout)

    def test_buildout_succeeds(self):
        status = self.buildout()
        self.assertEqual(0, status)
        self.assertEqual(
            ['buildout', 'doc', 'test'], sorted(os.listdir('bin')))

    def test_tests_succeed(self):
        self.buildout()
        bin_test = os.path.join('bin', 'test')
        self.assertTrue(os.path.isfile(bin_test))
        status = subprocess.call([bin_test])
        self.assertEqual(0, status)

    def test_sphinx_docs_can_be_built(self):
        self.buildout()
        bin_doc = os.path.join('bin', 'doc')
        self.assertTrue(os.path.isfile(bin_doc))
        subprocess.call([bin_doc])
        self.assertIn('<html', self.content('build/doc/index.html'))

    def test_sphinx_api_docs_are_updated_with_every_run(self):
        self.buildout()
        bin_doc = os.path.join('bin', 'doc')
        api_txt = open(os.path.join('doc', 'api.txt'), 'w')
        api_txt.write("""\
.. autosummary::
    :toctree: %s

    gocept.example.foo
""" % gocept.package.sphinxconf.AUTOSUMMARY_OUTPUT)
        api_txt.close()
        foo_py = open(os.path.join('src', 'gocept', 'example', 'foo.py'), 'w')
        foo_py.write('"A module doc string."')
        foo_py.close()
        subprocess.call([bin_doc])
        self.assertIn('A module doc string.',
                      self.content('build/doc/_api/gocept.example.foo.html'))
        # Make sure the mtime of both the Python source and the Sphinx input
        # files generated by autosummary is greater at a precision of 1 second
        # during the following run than the Sphinx output produced in the
        # previous run.
        time.sleep(1)
        foo_py = open(os.path.join('src', 'gocept', 'example', 'foo.py'), 'w')
        foo_py.write('def some_function(): pass')
        foo_py.close()
        subprocess.call([bin_doc])
        self.assertIn('some_function',
                      self.content('build/doc/_api/gocept.example.foo.html'))
